# -*- coding: utf-8 -*-
"""
Created on Tue Dec 22 10:10:42 2020

@author: connor o'brien
"""


__author__ = "Connor O'Brien"
__email__ = "obrienco@bu.edu"
__url__ = "https://github.com/Cupid-Cubesat/Cupid_Data_Start"
__license__ = "MIT"
__description__ = "CuPID function library."